# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "BudgetHistoryResponse",
    "BudgetHistory",
    "BudgetHistoryTotals",
    "BudgetHistoryTotalsCost",
    "BudgetHistoryTotalsCostInput",
    "BudgetHistoryTotalsCostOutput",
    "BudgetHistoryTotalsCostTotal",
    "BudgetHistoryTotalsRequests",
]


class BudgetHistoryTotalsCostInput(BaseModel):
    base: Optional[float] = None

    billed: Optional[float] = None

    overrun_base: Optional[float] = FieldInfo(alias="overrunBase", default=None)

    overrun_billed: Optional[float] = FieldInfo(alias="overrunBilled", default=None)

    revenue: Optional[float] = None


class BudgetHistoryTotalsCostOutput(BaseModel):
    base: Optional[float] = None

    billed: Optional[float] = None

    overrun_base: Optional[float] = FieldInfo(alias="overrunBase", default=None)

    overrun_billed: Optional[float] = FieldInfo(alias="overrunBilled", default=None)

    revenue: Optional[float] = None


class BudgetHistoryTotalsCostTotal(BaseModel):
    base: Optional[float] = None

    billed: Optional[float] = None

    overrun_base: Optional[float] = FieldInfo(alias="overrunBase", default=None)

    overrun_billed: Optional[float] = FieldInfo(alias="overrunBilled", default=None)

    revenue: Optional[float] = None


class BudgetHistoryTotalsCost(BaseModel):
    input: Optional[BudgetHistoryTotalsCostInput] = None

    output: Optional[BudgetHistoryTotalsCostOutput] = None

    total: Optional[BudgetHistoryTotalsCostTotal] = None


class BudgetHistoryTotalsRequests(BaseModel):
    blocked: Optional[int] = None

    exceeded: Optional[int] = None

    failed: Optional[int] = None

    successful: Optional[int] = None

    total: Optional[int] = None


class BudgetHistoryTotals(BaseModel):
    cost: Optional[BudgetHistoryTotalsCost] = None

    requests: Optional[BudgetHistoryTotalsRequests] = None


class BudgetHistory(BaseModel):
    base_cost_estimate: Optional[Literal["Max"]] = None

    budget_id: Optional[str] = None

    budget_name: Optional[str] = None

    budget_response_type: Optional[Literal["Block", "Allow"]] = None

    budget_tags: Optional[List[str]] = None

    budget_type: Optional[Literal["Conservative", "Liberal"]] = None

    max: Optional[float] = None

    reset_date: Optional[datetime] = None

    totals: Optional[BudgetHistoryTotals] = None


class BudgetHistoryResponse(BaseModel):
    budget_history: BudgetHistory

    request_id: str

    message: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .cost_data import CostData as CostData
from .cost_details import CostDetails as CostDetails
from .proxy_result import ProxyResult as ProxyResult
from .requests_data import RequestsData as RequestsData
from .budget_response import BudgetResponse as BudgetResponse
from .default_response import DefaultResponse as DefaultResponse
from .paged_budget_list import PagedBudgetList as PagedBudgetList
from .budget_list_params import BudgetListParams as BudgetListParams
from .ingest_units_params import IngestUnitsParams as IngestUnitsParams
from .budget_create_params import BudgetCreateParams as BudgetCreateParams
from .budget_update_params import BudgetUpdateParams as BudgetUpdateParams
from .budget_history_response import BudgetHistoryResponse as BudgetHistoryResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["RequestsData"]


class RequestsData(BaseModel):
    blocked: Optional[int] = None

    error: Optional[int] = None

    exceeded: Optional[int] = None

    failed: Optional[int] = None

    successful: Optional[int] = None

    total: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["IngestUnitsParams"]


class IngestUnitsParams(TypedDict, total=False):
    category: Required[str]

    input: Required[int]

    output: Required[int]

    resource: Required[str]

    budget_ids: Annotated[list[str], PropertyInfo(alias="xProxy-Budget-IDs")]

    request_tags: Annotated[list[str], PropertyInfo(alias="xProxy-Request-Tags")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import ingest_units_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    strip_not_given,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import (
    make_request_options,
)
from ..types.proxy_result import ProxyResult

__all__ = ["IngestResource", "AsyncIngestResource"]


class IngestResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IngestResourceWithRawResponse:
        return IngestResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IngestResourceWithStreamingResponse:
        return IngestResourceWithStreamingResponse(self)

    def units(
        self,
        *,
        category: str,
        input: int,
        output: int,
        resource: str,
        budget_ids: list[str] | NotGiven = NOT_GIVEN,
        request_tags: list[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProxyResult:
        """
        Ingest a request

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        valid_ids_str: str | NotGiven = NOT_GIVEN
        valid_tags_str: str | NotGiven = NOT_GIVEN

        if isinstance(budget_ids, NotGiven):
            valid_ids_str = NOT_GIVEN
        else:
            # Proceed with the list comprehension if budget_ids is not NotGiven
            valid_ids = [id.strip() for id in budget_ids if id.strip()]
            valid_ids_str = ",".join(valid_ids) if valid_ids else NOT_GIVEN
        if isinstance(request_tags, NotGiven):
            valid_tags_str = NOT_GIVEN
        else:
            # Proceed with the list comprehension if budget_ids is not NotGiven
            valid_tags = [tag.strip() for tag in request_tags if tag.strip()]
            valid_tags_str = ",".join(valid_tags) if valid_tags else NOT_GIVEN

        extra_headers = {
            **strip_not_given(
                {
                    "xProxy-Budget-IDs": valid_ids_str,
                    "xProxy-Request-Tags": valid_tags_str,
                }
            ),
            **(extra_headers or {}),
        }
        return self._post(
            "/api/v1/ingest",
            body=maybe_transform(
                {
                    "category": category,
                    "input": input,
                    "output": output,
                    "resource": resource,
                },
                ingest_units_params.IngestUnitsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyResult,
        )


class AsyncIngestResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIngestResourceWithRawResponse:
        return AsyncIngestResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIngestResourceWithStreamingResponse:
        return AsyncIngestResourceWithStreamingResponse(self)

    async def units(
        self,
        *,
        category: str,
        input: int,
        output: int,
        resource: str,
        budget_ids: list[str] | NotGiven = NOT_GIVEN,
        request_tags: list[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProxyResult:
        """
        Ingest a request

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        valid_ids_str: str | NotGiven = NOT_GIVEN
        valid_tags_str: str | NotGiven = NOT_GIVEN

        if isinstance(budget_ids, NotGiven):
            valid_ids_str = NOT_GIVEN
        else:
            # Proceed with the list comprehension if budget_ids is not NotGiven
            valid_ids = [id.strip() for id in budget_ids if id.strip()]
            valid_ids_str = ",".join(valid_ids) if valid_ids else NOT_GIVEN
        if isinstance(request_tags, NotGiven):
            valid_tags_str = NOT_GIVEN
        else:
            # Proceed with the list comprehension if budget_ids is not NotGiven
            valid_tags = [tag.strip() for tag in request_tags if tag.strip()]
            valid_tags_str = ",".join(valid_tags) if valid_tags else NOT_GIVEN

        extra_headers = {
            **strip_not_given(
                {
                    "xProxy-Budget-IDs": valid_ids_str,
                    "xProxy-Request-Tags": valid_tags_str,
                }
            ),
            **(extra_headers or {}),
        }
        return await self._post(
            "/api/v1/ingest",
            body=await async_maybe_transform(
                {
                    "category": category,
                    "input": input,
                    "output": output,
                    "resource": resource,
                },
                ingest_units_params.IngestUnitsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyResult,
        )


class IngestResourceWithRawResponse:
    def __init__(self, ingest: IngestResource) -> None:
        self._ingest = ingest

        self.units = to_raw_response_wrapper(
            ingest.units,
        )


class AsyncIngestResourceWithRawResponse:
    def __init__(self, ingest: AsyncIngestResource) -> None:
        self._ingest = ingest

        self.units = async_to_raw_response_wrapper(
            ingest.units,
        )


class IngestResourceWithStreamingResponse:
    def __init__(self, ingest: IngestResource) -> None:
        self._ingest = ingest

        self.units = to_streamed_response_wrapper(
            ingest.units,
        )


class AsyncIngestResourceWithStreamingResponse:
    def __init__(self, ingest: AsyncIngestResource) -> None:
        self._ingest = ingest

        self.units = async_to_streamed_response_wrapper(
            ingest.units,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BudgetListParams"]


class BudgetListParams(TypedDict, total=False):
    budget_name: Annotated[str, PropertyInfo(alias="BudgetName")]

    page_number: Annotated[int, PropertyInfo(alias="PageNumber")]

    page_size: Annotated[int, PropertyInfo(alias="PageSize")]

    sort_ascending: Annotated[bool, PropertyInfo(alias="SortAscending")]

    sort_by: Annotated[str, PropertyInfo(alias="SortBy")]

    tags: Annotated[str, PropertyInfo(alias="Tags")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["BudgetCreateParams"]


class BudgetCreateParams(TypedDict, total=False):
    budget_name: Required[str]

    max: Required[float]

    base_cost_estimate: Literal["Max"]

    budget_response_type: Literal["Block", "Allow"]

    budget_tags: Optional[List[str]]

    budget_type: Literal["Conservative", "Liberal"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .ingest import (
    IngestResource,
    AsyncIngestResource,
    IngestResourceWithRawResponse,
    AsyncIngestResourceWithRawResponse,
    IngestResourceWithStreamingResponse,
    AsyncIngestResourceWithStreamingResponse,
)
from .budgets import (
    BudgetsResource,
    AsyncBudgetsResource,
    BudgetsResourceWithRawResponse,
    AsyncBudgetsResourceWithRawResponse,
    BudgetsResourceWithStreamingResponse,
    AsyncBudgetsResourceWithStreamingResponse,
)

__all__ = [
    "BudgetsResource",
    "AsyncBudgetsResource",
    "BudgetsResourceWithRawResponse",
    "AsyncBudgetsResourceWithRawResponse",
    "BudgetsResourceWithStreamingResponse",
    "AsyncBudgetsResourceWithStreamingResponse",
    "IngestResource",
    "AsyncIngestResource",
    "IngestResourceWithRawResponse",
    "AsyncIngestResourceWithRawResponse",
    "IngestResourceWithStreamingResponse",
    "AsyncIngestResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .cost_data import CostData
from .requests_data import RequestsData

__all__ = ["BudgetResponse", "Budget", "BudgetTotals"]


class BudgetTotals(BaseModel):
    cost: Optional[CostData] = None

    requests: Optional[RequestsData] = None


class Budget(BaseModel):
    base_cost_estimate: Literal["Max"]

    budget_creation_timestamp: datetime

    budget_id: str

    budget_name: str

    budget_response_type: Literal["Block", "Allow"]

    budget_type: Literal["Conservative", "Liberal"]

    budget_update_timestamp: datetime

    currency: str

    max: float

    totals: BudgetTotals

    budget_tags: Optional[List[str]] = None


class BudgetResponse(BaseModel):
    budget: Budget

    request_id: str

    message: Optional[str] = None
